# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './src/app/gui/dlg/selectconnectiondialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SelectionConnectionDialog(object):
    def setupUi(self, SelectionConnectionDialog):
        SelectionConnectionDialog.setObjectName("SelectionConnectionDialog")
        SelectionConnectionDialog.resize(461, 103)
        SelectionConnectionDialog.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(SelectionConnectionDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(SelectionConnectionDialog)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.comboBox = QtWidgets.QComboBox(SelectionConnectionDialog)
        self.comboBox.setObjectName("comboBox")
        self.verticalLayout.addWidget(self.comboBox)
        spacerItem = QtWidgets.QSpacerItem(20, 27, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(SelectionConnectionDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(SelectionConnectionDialog)
        self.buttonBox.accepted.connect(SelectionConnectionDialog.accept)
        self.buttonBox.rejected.connect(SelectionConnectionDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(SelectionConnectionDialog)

    def retranslateUi(self, SelectionConnectionDialog):
        _translate = QtCore.QCoreApplication.translate
        SelectionConnectionDialog.setWindowTitle(_translate("SelectionConnectionDialog", "Dsmcp - Select remote connection"))
        self.label.setText(_translate("SelectionConnectionDialog", "Select remote connection"))
