# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './src/app/gui/basicwindow.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_BasicWindow(object):
    def setupUi(self, BasicWindow):
        BasicWindow.setObjectName("BasicWindow")
        BasicWindow.resize(354, 456)
        self.centralwidget = QtWidgets.QWidget(BasicWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.graphicsViewPanel = QSvgPanel(self.centralwidget)
        self.graphicsViewPanel.setObjectName("graphicsViewPanel")
        self.gridLayout.addWidget(self.graphicsViewPanel, 0, 0, 1, 2)
        self.pushButtonStart = QActionButton(self.centralwidget)
        self.pushButtonStart.setObjectName("pushButtonStart")
        self.gridLayout.addWidget(self.pushButtonStart, 2, 0, 1, 1)
        self.pushButtonStop = QActionButton(self.centralwidget)
        self.pushButtonStop.setObjectName("pushButtonStop")
        self.gridLayout.addWidget(self.pushButtonStop, 2, 1, 1, 1)
        self.widgetSpeed = QSpeed(self.centralwidget)
        self.widgetSpeed.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.widgetSpeed.setFrameShadow(QtWidgets.QFrame.Raised)
        self.widgetSpeed.setObjectName("widgetSpeed")
        self.gridLayout.addWidget(self.widgetSpeed, 3, 0, 1, 2)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 4, 0, 1, 2)
        BasicWindow.setCentralWidget(self.centralwidget)
        self.menuBar = QtWidgets.QMenuBar(BasicWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 354, 22))
        self.menuBar.setObjectName("menuBar")
        self.menuSystem = QtWidgets.QMenu(self.menuBar)
        self.menuSystem.setObjectName("menuSystem")
        self.menuMotor = QtWidgets.QMenu(self.menuBar)
        self.menuMotor.setObjectName("menuMotor")
        self.menuHelp = QtWidgets.QMenu(self.menuBar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuWorkspace = QtWidgets.QMenu(self.menuBar)
        self.menuWorkspace.setObjectName("menuWorkspace")
        self.menuBoard = QtWidgets.QMenu(self.menuBar)
        self.menuBoard.setObjectName("menuBoard")
        BasicWindow.setMenuBar(self.menuBar)
        self.statusBar = QtWidgets.QStatusBar(BasicWindow)
        self.statusBar.setObjectName("statusBar")
        BasicWindow.setStatusBar(self.statusBar)
        self.actionSysConnect = QtWidgets.QAction(BasicWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/img/link.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionSysConnect.setIcon(icon)
        self.actionSysConnect.setObjectName("actionSysConnect")
        self.actionMotorStart = QtWidgets.QAction(BasicWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/img/start.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionMotorStart.setIcon(icon1)
        self.actionMotorStart.setObjectName("actionMotorStart")
        self.actionLightEnabled = QtWidgets.QAction(BasicWindow)
        self.actionLightEnabled.setCheckable(True)
        self.actionLightEnabled.setChecked(True)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/img/light.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionLightEnabled.setIcon(icon2)
        self.actionLightEnabled.setObjectName("actionLightEnabled")
        self.actionLightBlue = QtWidgets.QAction(BasicWindow)
        self.actionLightBlue.setCheckable(True)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/img/light-blue.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionLightBlue.setIcon(icon3)
        self.actionLightBlue.setObjectName("actionLightBlue")
        self.actionMotorCCW = QtWidgets.QAction(BasicWindow)
        self.actionMotorCCW.setCheckable(True)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/img/counterclockwise.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionMotorCCW.setIcon(icon4)
        self.actionMotorCCW.setObjectName("actionMotorCCW")
        self.actionAbout = QtWidgets.QAction(BasicWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionQuit = QtWidgets.QAction(BasicWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionAutoRefresh = QtWidgets.QAction(BasicWindow)
        self.actionAutoRefresh.setCheckable(True)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/img/autorefresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionAutoRefresh.setIcon(icon5)
        self.actionAutoRefresh.setObjectName("actionAutoRefresh")
        self.actionSysDisconnect = QtWidgets.QAction(BasicWindow)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/img/unlink.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionSysDisconnect.setIcon(icon6)
        self.actionSysDisconnect.setObjectName("actionSysDisconnect")
        self.actionRefresh = QtWidgets.QAction(BasicWindow)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/img/refresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionRefresh.setIcon(icon7)
        self.actionRefresh.setObjectName("actionRefresh")
        self.actionHelp = QtWidgets.QAction(BasicWindow)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/img/help.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionHelp.setIcon(icon8)
        self.actionHelp.setObjectName("actionHelp")
        self.actionMotorCW = QtWidgets.QAction(BasicWindow)
        self.actionMotorCW.setCheckable(True)
        self.actionMotorCW.setChecked(True)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/img/clockwise.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionMotorCW.setIcon(icon9)
        self.actionMotorCW.setObjectName("actionMotorCW")
        self.actionMotorStop = QtWidgets.QAction(BasicWindow)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/img/stop.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionMotorStop.setIcon(icon10)
        self.actionMotorStop.setObjectName("actionMotorStop")
        self.actionFirmReset = QtWidgets.QAction(BasicWindow)
        self.actionFirmReset.setObjectName("actionFirmReset")
        self.actionAPISendFirmware = QtWidgets.QAction(BasicWindow)
        self.actionAPISendFirmware.setObjectName("actionAPISendFirmware")
        self.actionAboutBoard = QtWidgets.QAction(BasicWindow)
        self.actionAboutBoard.setObjectName("actionAboutBoard")
        self.actionMotorMode = QtWidgets.QAction(BasicWindow)
        self.actionMotorMode.setObjectName("actionMotorMode")
        self.menuSystem.addAction(self.actionRefresh)
        self.menuSystem.addAction(self.actionAutoRefresh)
        self.menuSystem.addSeparator()
        self.menuSystem.addAction(self.actionQuit)
        self.menuMotor.addAction(self.actionMotorMode)
        self.menuMotor.addAction(self.actionMotorStart)
        self.menuMotor.addAction(self.actionMotorStop)
        self.menuMotor.addAction(self.actionLightEnabled)
        self.menuMotor.addAction(self.actionLightBlue)
        self.menuMotor.addAction(self.actionMotorCCW)
        self.menuHelp.addAction(self.actionHelp)
        self.menuHelp.addAction(self.actionAbout)
        self.menuBoard.addAction(self.actionFirmReset)
        self.menuBoard.addAction(self.actionAPISendFirmware)
        self.menuBoard.addAction(self.actionAboutBoard)
        self.menuBar.addAction(self.menuSystem.menuAction())
        self.menuBar.addAction(self.menuBoard.menuAction())
        self.menuBar.addAction(self.menuWorkspace.menuAction())
        self.menuBar.addAction(self.menuMotor.menuAction())
        self.menuBar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(BasicWindow)
        self.actionQuit.triggered.connect(BasicWindow.close)
        QtCore.QMetaObject.connectSlotsByName(BasicWindow)

    def retranslateUi(self, BasicWindow):
        _translate = QtCore.QCoreApplication.translate
        BasicWindow.setWindowTitle(_translate("BasicWindow", "DSMCP"))
        self.pushButtonStart.setText(_translate("BasicWindow", "Start"))
        self.pushButtonStop.setText(_translate("BasicWindow", "Stop"))
        self.label.setText(_translate("BasicWindow", "TextLabel"))
        self.menuSystem.setTitle(_translate("BasicWindow", "System"))
        self.menuMotor.setTitle(_translate("BasicWindow", "Motor"))
        self.menuHelp.setTitle(_translate("BasicWindow", "Help"))
        self.menuWorkspace.setTitle(_translate("BasicWindow", "Workspace"))
        self.menuBoard.setTitle(_translate("BasicWindow", "Board"))
        self.actionSysConnect.setText(_translate("BasicWindow", "Connect"))
        self.actionSysConnect.setToolTip(_translate("BasicWindow", "Connect DCP to board"))
        self.actionMotorStart.setText(_translate("BasicWindow", "Start"))
        self.actionMotorStart.setToolTip(_translate("BasicWindow", "Start the motor"))
        self.actionMotorStart.setShortcut(_translate("BasicWindow", "F2"))
        self.actionLightEnabled.setText(_translate("BasicWindow", "Light Enabled"))
        self.actionLightEnabled.setToolTip(_translate("BasicWindow", "Enable/disable Light"))
        self.actionLightEnabled.setShortcut(_translate("BasicWindow", "F4"))
        self.actionLightBlue.setText(_translate("BasicWindow", "BlueLight"))
        self.actionLightBlue.setToolTip(_translate("BasicWindow", "Switch white / blue light"))
        self.actionLightBlue.setShortcut(_translate("BasicWindow", "F9"))
        self.actionMotorCCW.setText(_translate("BasicWindow", "CCW"))
        self.actionMotorCCW.setToolTip(_translate("BasicWindow", "Set rotation to counter clockwise direction"))
        self.actionMotorCCW.setShortcut(_translate("BasicWindow", "F7"))
        self.actionAbout.setText(_translate("BasicWindow", "About ..."))
        self.actionAbout.setShortcut(_translate("BasicWindow", "Ctrl+F1"))
        self.actionQuit.setText(_translate("BasicWindow", "Quit"))
        self.actionQuit.setShortcut(_translate("BasicWindow", "Ctrl+Q"))
        self.actionAutoRefresh.setText(_translate("BasicWindow", "AutoRefresh"))
        self.actionSysDisconnect.setText(_translate("BasicWindow", "Disconnect"))
        self.actionSysDisconnect.setToolTip(_translate("BasicWindow", "Disconnect DCP from board"))
        self.actionRefresh.setText(_translate("BasicWindow", "Refresh"))
        self.actionRefresh.setToolTip(_translate("BasicWindow", "Refresh"))
        self.actionRefresh.setShortcut(_translate("BasicWindow", "F5"))
        self.actionHelp.setText(_translate("BasicWindow", "Help"))
        self.actionHelp.setShortcut(_translate("BasicWindow", "F1"))
        self.actionMotorCW.setText(_translate("BasicWindow", "CW"))
        self.actionMotorCW.setToolTip(_translate("BasicWindow", "Set rotation to clockwise direction"))
        self.actionMotorCW.setShortcut(_translate("BasicWindow", "F6"))
        self.actionMotorStop.setText(_translate("BasicWindow", "Stop"))
        self.actionMotorStop.setToolTip(_translate("BasicWindow", "Stop the motor"))
        self.actionMotorStop.setShortcut(_translate("BasicWindow", "F3"))
        self.actionFirmReset.setText(_translate("BasicWindow", "Reset"))
        self.actionAPISendFirmware.setText(_translate("BasicWindow", "Send Firmware"))
        self.actionAboutBoard.setText(_translate("BasicWindow", "About board..."))
        self.actionMotorMode.setText(_translate("BasicWindow", "Motor mode..."))
        self.actionMotorMode.setToolTip(_translate("BasicWindow", "Show motor mode dialog"))
        self.actionMotorMode.setShortcut(_translate("BasicWindow", "F10"))
from app.gui.widget.qactionbutton import QActionButton
from app.gui.widget.qspeed import QSpeed
from app.gui.widget.qsvgpanel import QSvgPanel
