'''

:author:  F. Voillat
:date: 2021-05-10 Creation
:copyright: Dassym SA 2021
'''
from dboard import SystemModeConfig

SystemModeConfigIcon = {
    SystemModeConfig.START : None,
    SystemModeConfig.HOLDING : None,
    SystemModeConfig.ROCKING : ':/img/rocking.svg',
    SystemModeConfig.AUTOBOOST : None,
    SystemModeConfig.AUTOSTOP : ':/img/autostop.svg',
    SystemModeConfig.AUTOREVERSE : ':/img/autoreverse.svg',
    SystemModeConfig.REVERSE : ':/img/reverse.svg',
    SystemModeConfig.QUADRATIC : ':img/quadratic.svg',
    SystemModeConfig.LIGHT : ':/img/light.svg',
    SystemModeConfig.FREEWHEEL : ':/img/freewheel.svg',
    SystemModeConfig.INDIRECT : ':/img/indirect.svg',
    SystemModeConfig.LIGHTAUTO : ':img/lightauto.svg',
    }